import { LightningElement, api, track, wire } from 'lwc';
import { loadStyle, loadScript } from 'lightning/platformResourceLoader';
import GanttFiles from '@salesforce/resourceUrl/SparQ_Gantt';


export default class GanttPOC extends LightningElement {
    ganttInitialized = false;

    renderedCallback() {
        if (this.ganttInitialized) {
            return;
        }
        this.ganttInitialized = true;
        Promise.all([
            loadScript(this, GanttFiles + '/codebase/dhtmlxgantt.js'),
            loadStyle(this, GanttFiles + '/codebase/dhtmlxgantt.css'),
        ]).then(() => {
            this.initializeUI();
        }).catch(error => {
            //showToastMessage('Error loading Gantt', error.body.message, 'error');
        });
    }

    async initializeUI() {
        const root = this.template.querySelector('.gantt_here');
        root.style.height = "400px";
        gantt = window.Gantt.getGanttInstance();

        gantt.config.show_errors = false;

        //Enable any plugin here
        gantt.plugins({
            marker: true,
            keyboard_navigation: true,
            fullscreen: true
        });

        //Adding a marker at today's date
        var date_to_str = gantt.date.date_to_str(gantt.config.task_date);
        var nowDate = new Date();
        //Remove time part from date to fix marker position
        var today_Date = new Date(nowDate.getFullYear(), nowDate.getMonth(), nowDate.getDate());
        var id = gantt.addMarker({ start_date: today_Date, css: "today", text: "Today Date", title: "Today Date: " + date_to_str(today_Date) });
        //=========================================================================================
        //Configrable Grid layout
        // gantt.config.layout = {
        //     css: "gantt_container",
        //     cols: [
        //         {
        //             width: 370, min_width: 200,
        //             rows:
        //                 [
        //                     { view: "grid", scrollX: "gridScroll", scrollable: true, scrollY: "scrollVer" },
        //                     { view: "scrollbar", id: "gridScroll" }
        //                 ]
        //         },
        //         { resizer: true, width: 1 },
        //         {
        //             rows: [
        //                 { view: "timeline", scrollX: "scrollHor", scrollY: "scrollVer" },
        //                 { view: "scrollbar", id: "scrollHor" }
        //             ]
        //         },
        //         { view: "scrollbar", id: "scrollVer" }
        //     ]
        // };

        gantt.config.order_branch = "marker";        
        gantt.config.order_branch_free = true;       
        gantt.config.show_tasks_outside_timescale = true;
        //gantt.config.grid_elastic_columns = true;        
        gantt.config.autosize = "y"; 
        //gantt.config.min_column_width = 100;
        gantt.config.keep_grid_width = false;
        gantt.config.grid_resize = true;
        gantt.config.scale_height = 20 * 3;
        gantt.config.show_unscheduled = true;

        var zoomConfig = {
            minColumnWidth: 50,
            maxColumnWidth: 150,
            levels: [

                [
                    {
                        unit: "year", format: "%Y", step: 1
                    }
                ],
                [
                    {
                        unit: "month", step: 1, format: "%M"
                    },
                    {
                        unit: "quarter", step: 1, format: function (date) {
                            var dateToStr = gantt.date.date_to_str("%M");
                            var endDate = gantt.date.add(gantt.date.add(date, 3, "month"), -1, "day");
                            return dateToStr(date) + " - " + dateToStr(endDate);
                        }
                    }
                ],
                [
                    {
                        unit: "month", format: "%M %Y", step: 1
                    },
                    {
                        unit: "week", step: 1, format: function (date) {
                            var dateToStr = gantt.date.date_to_str("%d %M");
                            var endDate = gantt.date.add(date, +6, "day");
                            var weekNum = gantt.date.date_to_str("%W")(date);
                            return "Week #" + weekNum + ", " + dateToStr(date) + " - " + dateToStr(endDate);
                        }
                    }
                ],
                [
                    {
                        unit: "month", format: "%M %Y", step: 1
                    },
                    {
                        unit: "day", format: "%d %M", step: 1
                    }
                ]
                
            ]
            ,
            startDate: gantt.config.start_date,
            endDate: gantt.config.end_date,
            useKey: "ctrlKey",
            trigger: "wheel",
            element: function () {
                return gantt.$root.querySelector(".gantt_task");
            }
        }


        gantt.ext.zoom.init(zoomConfig);

        gantt.init(root);

        var data = {
            "data":[
                {"id":"10", "text":"Project #1", "start_date":"01-04-2025", "duration":3, "order":10,"progress":0.5, "open": true},
                {"id":"1", "text":"Task #1",    "start_date":"02-04-2025", "duration":2,  "order":10,"progress":0.2, "parent":"10"},
                {"id":"2", "text":"Task #2",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.2, "parent":"10"},
                {"id":"20", "text":"Project #2", "start_date":"01-04-2025", "duration":3, "order":10,"progress":0.5, "type": "project", "open": true},
                {"id":"3", "text":"Task #3",    "start_date":"02-04-2025", "duration":2,  "order":10,"progress":0.8, "parent":"20"},
                {"id":"4", "text":"Task #4",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.8, "parent":"20"},
              {"id":"5", "text":"Task #5",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.8, "parent":"20"},
              {"id":"6", "text":"Task #6",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.8, "parent":"20"},
              {"id":"7", "text":"Task #7",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.8, "parent":"20"},
              {"id":"8", "text":"Task #8",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.8, "parent":"20"},
              {"id":"9", "text":"Task #9",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.8, "parent":"20"},
              {"id":"10", "text":"Task #10",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.8, "parent":"20"},
              {"id":"11", "text":"Task #11",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.8, "parent":"20"},
              {"id":"12", "text":"Task #12",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.8},
              {"id":"13", "text":"Task #13",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.8},
              {"id":"14", "text":"Task #14",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.8},
              {"id":"15", "text":"Task #15",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.8},
              {"id":"16", "text":"Task #16",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.8},
              {"id":"17", "text":"Task #17",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":1.2},
              {"id":"18", "text":"Task #18",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":0.9},
              {"id":"19", "text":"Task #19",    "start_date":"01-04-2025", "duration":2,  "order":20,"progress":1}
              
            ],
            "links":[
                { "id":1, "source":1, "target":2, "type":"0"},
                { "id":2, "source":2, "target":3, "type":"0"},
                { "id":3, "source":3, "target":4, "type":"0"},
                { "id":4, "source":2, "target":5, "type":"0"}
            ]
        };

        gantt.parse(data);
    }
}